@@ -15,7 +15,9 @@ from .block import Block
 from .device import Device
 from .utils import exception_log, timer
 from .coap import CoAP
+from .mqtt import MQTT
 from .mdns import MDns
+from .firmware import Firmware_manager
 
 #from .device.relay import Relay
 #from .device.switch import Switch
@@ -40,6 +42,7 @@ from .const import (
     INFO_VALUE_SSID,
     INFO_VALUE_HAS_FIRMWARE_UPDATE,
     INFO_VALUE_LATEST_FIRMWARE_VERSION,
+    INFO_VALUE_LATEST_BETA_FW_VERSION,
     INFO_VALUE_FW_VERSION,
     INFO_VALUE_CLOUD_STATUS,
     INFO_VALUE_CLOUD_ENABLED,
@@ -58,11 +61,6 @@ from .const import (
 
 __version__ = VERSION
 
-try:
-    import http.client as httplib
-except:
-    import httplib
-
 class pyShelly():
     def __init__(self, loop=None):
         LOGGER.info("Init  %s", VERSION)
@@ -91,8 +89,13 @@ class pyShelly():
 
         self._coap = CoAP(self)
         self._mdns = None
+        self._mqtt = MQTT(self)
+        self._firmware_mgr =  Firmware_manager(self)
         self.host_ip = ''
         self.bind_ip = ''
+        self.mqtt_port = 0
+        self.firmware_url = None
+        self.zeroconf = None
 
         self._shelly_by_ip = {}
         #self.loop = asyncio.get_event_loop()
@@ -123,6 +126,8 @@ class pyShelly():
             self._coap.start()
         if self._mdns:
             self._mdns.start()
+        if self._mqtt:
+            self._mqtt.start()
         #asyncio.ensure_future(self._update_loop())
         self._update_thread = threading.Thread(target=self._update_loop)
         self._update_thread.name = "Poll"
@@ -139,7 +144,9 @@ class pyShelly():
         if self._coap:
             self._coap.close()
         if self._mdns:
-            self._mdns.close()
+            self._mdns.close(self.zeroconf)
+        if self._mqtt:
+            self._mqtt.close()
         if self._update_thread is not None:
             self._update_thread.join()
         if self._socket:
@@ -175,6 +182,10 @@ class pyShelly():
                         done = True
                         self._shelly_by_ip[ip_addr]['done'] = True
                         dev = settings["device"]
+                        if not "hostname" in dev: #invalide device
+                            LOGGER.info("Error adding device (missing hostname), %s %s",
+                                ip_addr, data['src'])
+                            continue
                         device_id = dev["hostname"].rpartition('-')[2]
                         device_type = dev["type"]
                         ip_addr = status["wifi_sta"]["ip"]
@@ -211,13 +222,16 @@ class pyShelly():
         for callback in self.cb_device_removed:
             callback(dev, discovery_src)
 
-    def update_block(self, block_id, device_type, ipaddr, src, payload):
+    def update_block(self, block_id, device_type, ipaddr, src, payload,
+                     force_poll=False):
         if self.only_device_id is not None and \
             block_id != self.only_device_id:
             return
 
         block_added = False
         if block_id not in self.blocks:
+            if not ipaddr:
+                return
             block = self.blocks[block_id] = \
                 Block(self, block_id, device_type, ipaddr, src)
             block_added = True
@@ -229,7 +243,7 @@ class pyShelly():
 
         if payload:
             data = {d[1]:d[2] for d in json.loads(payload)['G']}
-            block.update(data, ipaddr)
+            block.update_coap(data, ipaddr)
             block.payload = payload
 
         if block_added:
@@ -238,8 +252,8 @@ class pyShelly():
             for device in block.devices:
                 self.add_device(device, block.discovery_src)
 
-        if block.sleep_device:
-            self._poll_block(block)
+        if block.sleep_device or force_poll:
+            self._poll_block(block, force_poll)
 
     def _update_loop(self):
         LOGGER.debug("Start update loop, %s sec", self.update_status_interval)
@@ -258,17 +272,19 @@ class pyShelly():
                         self._poll_block(block)
 
                     block.check_available()
+                    block.loop()
 
-                time.sleep(1)
+                time.sleep(10)
             except Exception as ex:
                 exception_log(ex, "Error update loop")
 
-    def _poll_block(self, block):
+    def _poll_block(self, block, force=False):
         now = datetime.now()
-        if self.update_status_interval is not None and \
+        if force or \
+            (self.update_status_interval is not None and \
             (block.last_update_status_info is None or \
             now - block.last_update_status_info \
-                > self.update_status_interval):
+                > self.update_status_interval)):
 
             LOGGER.debug("Polling block, %s %s", block.id, block.type)
             block.last_update_status_info = now
