@@ -14,7 +14,6 @@ from .const import (
     COAP_IP,
     COAP_PORT
 )
-
 class CoAP():
 
     def __init__(self, root):
@@ -169,7 +168,13 @@ class CoAP():
 
                         byte = data[pos]
 
-                    payload = s(data[pos + 1:])
+                    try:
+                        payload = s(data[pos + 1:])
+                    except:
+                        print(data[pos + 1:])
+
+                    if payload: #Fix for DW2 payload error
+                        payload = payload.replace(",,",",").replace("][", "],[")
 
                     LOGGER.debug('CoAP Code: %s, Type %s, Id %s, Payload *%s*',
                                  code, device_type, device_id,
@@ -184,5 +189,6 @@ class CoAP():
                                            ipaddr, 'CoAP-discovery', None)
 
             except Exception as ex:
-                LOGGER.debug("Error receive CoAP %s", str(ex))
+                #LOGGER.debug("Error receive CoAP %s", str(ex))
+                LOGGER.exception("Error receive CoAP")
                 #exception_log(ex, "Error receiving CoAP UDP")
